"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SmartFeedAccessory = void 0;
const hap_1 = require("./hap");
const operators_1 = require("rxjs/operators");
const rxjs_1 = require("rxjs");
const util_1 = require("./util");
class SmartFeedAccessory {
    constructor(feeder, accessory, { amount = 0 } = { amount: 0 }) {
        this.feeder = feeder;
        this.accessory = accessory;
        this.onCurrentlyFeeding = new rxjs_1.BehaviorSubject(false);
        const { Service, Characteristic } = hap_1.hap, feedService = this.getService(Service.Switch), feedCharacteristic = feedService.getCharacteristic(Characteristic.On), batteryService = this.getService(Service.BatteryService), accessoryInfoService = this.getService(Service.AccessoryInformation);
        this.registerCharacteristic(feedCharacteristic, this.onCurrentlyFeeding, (value) => __awaiter(this, void 0, void 0, function* () {
            if (value && !this.onCurrentlyFeeding.getValue()) {
                try {
                    this.onCurrentlyFeeding.next(true);
                    if (amount && amount > 0) {
                        yield feeder.feed({ amount });
                    }
                    else {
                        yield feeder.repeatLastFeed();
                    }
                    (0, util_1.logInfo)(`Done Feeding ${feeder.name}`);
                    yield (0, util_1.delay)(2 * 60 * 1000);
                }
                catch (e) {
                    (0, util_1.logError)('Failed to feed');
                    (0, util_1.logError)(e);
                }
                finally {
                    feedCharacteristic.updateValue(false);
                    this.onCurrentlyFeeding.next(false);
                }
            }
            else if (!value) {
                this.onCurrentlyFeeding.next(false);
            }
        }));
        this.registerCharacteristic(batteryService.getCharacteristic(Characteristic.BatteryLevel), feeder.onBatteryLevel);
        this.registerCharacteristic(batteryService.getCharacteristic(Characteristic.StatusLowBattery), feeder.onBatteryLevel.pipe((0, operators_1.map)((batteryLevel) => (batteryLevel < 20 ? 1 : 0))));
        this.registerCharacteristic(batteryService.getCharacteristic(Characteristic.ChargingState), feeder.onState.pipe((0, operators_1.map)((state) => (state.is_adapter_installed ? 1 : 0))));
        accessoryInfoService
            .getCharacteristic(Characteristic.Manufacturer)
            .updateValue('PetSafe');
        accessoryInfoService
            .getCharacteristic(Characteristic.Model)
            .updateValue(feeder.state.product_name || 'Smart Feed');
        accessoryInfoService
            .getCharacteristic(Characteristic.SerialNumber)
            .updateValue(feeder.id.toString());
        this.registerCharacteristic(accessoryInfoService.getCharacteristic(Characteristic.FirmwareRevision), feeder.onState.pipe((0, operators_1.map)((state) => state.firmware_version)));
        this.registerCharacteristic(accessoryInfoService.getCharacteristic(Characteristic.Name), feeder.onState.pipe((0, operators_1.map)((state) => state.settings.friendly_name)));
    }
    getService(serviceType) {
        const existingService = this.accessory.getService(serviceType);
        return existingService || this.accessory.addService(serviceType);
    }
    registerCharacteristic(characteristic, onValue, setValue) {
        const getValue = () => onValue.pipe((0, operators_1.take)(1)).toPromise();
        characteristic.on("get" /* CharacteristicEventTypes.GET */, (callback) => __awaiter(this, void 0, void 0, function* () {
            this.feeder.requestInfoUpdate();
            callback(null, yield getValue());
        }));
        if (setValue) {
            characteristic.on("set" /* CharacteristicEventTypes.SET */, (value, callback) => __awaiter(this, void 0, void 0, function* () {
                callback();
                const currentValue = yield getValue();
                if (value !== currentValue) {
                    setValue(value);
                }
            }));
        }
        onValue.pipe((0, operators_1.distinctUntilChanged)()).subscribe((value) => {
            characteristic.updateValue(value);
        });
    }
}
exports.SmartFeedAccessory = SmartFeedAccessory;
