"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.refreshTokens = exports.getAuthTokens = exports.initiateAuth = void 0;
const client_cognito_identity_provider_1 = require("@aws-sdk/client-cognito-identity-provider");
const PETSAFE_CLIENT_ID = '18hpp04puqmgf5nc6o474lcp2g';
let sharedClient;
function getClient() {
    if (!sharedClient) {
        sharedClient = new client_cognito_identity_provider_1.CognitoIdentityProviderClient({ region: 'us-east-1' });
    }
    return sharedClient;
}
function initiateAuth(email) {
    const client = getClient(), command = new client_cognito_identity_provider_1.InitiateAuthCommand({
        AuthFlow: client_cognito_identity_provider_1.AuthFlowType.CUSTOM_AUTH,
        AuthParameters: {
            USERNAME: email,
            AuthFlow: client_cognito_identity_provider_1.ChallengeNameType.CUSTOM_CHALLENGE,
        },
        ClientId: PETSAFE_CLIENT_ID,
    });
    return client.send(command);
}
exports.initiateAuth = initiateAuth;
function getAuthTokens(initiateAuthOutput, code) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = getClient(), command = new client_cognito_identity_provider_1.RespondToAuthChallengeCommand({
            ClientId: PETSAFE_CLIENT_ID,
            ChallengeName: initiateAuthOutput.ChallengeName,
            Session: initiateAuthOutput.Session,
            ChallengeResponses: {
                ANSWER: code.replace(/\D/g, ''),
                USERNAME: initiateAuthOutput.ChallengeParameters.USERNAME,
            },
        }), output = yield client.send(command);
        if (output.Session) {
            initiateAuthOutput.Session = output.Session;
        }
        return output;
    });
}
exports.getAuthTokens = getAuthTokens;
function refreshTokens(refreshToken) {
    const client = getClient(), command = new client_cognito_identity_provider_1.InitiateAuthCommand({
        AuthFlow: client_cognito_identity_provider_1.AuthFlowType.REFRESH_TOKEN_AUTH,
        AuthParameters: {
            REFRESH_TOKEN: refreshToken,
        },
        ClientId: PETSAFE_CLIENT_ID,
    });
    return client.send(command);
}
exports.refreshTokens = refreshTokens;
