"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-console */
const auth_1 = require("../auth");
const plugin_ui_utils_1 = require("@homebridge/plugin-ui-utils");
let initiateAuthOutput;
class PluginUiServer extends plugin_ui_utils_1.HomebridgePluginUiServer {
    constructor() {
        super();
        this.generateCode = ({ email }) => __awaiter(this, void 0, void 0, function* () {
            console.log(`Sending code to ${email}`);
            try {
                initiateAuthOutput = yield (0, auth_1.initiateAuth)(email);
            }
            catch (e) {
                console.error('Failed to send code to ' + email);
                console.error(e);
                throw new plugin_ui_utils_1.RequestError(e.message, e);
            }
        });
        this.generateToken = ({ email, code }) => __awaiter(this, void 0, void 0, function* () {
            console.log(`Getting token for ${email} with code ${code}`);
            if (!initiateAuthOutput) {
                console.error('Auth Initiation Not Found');
                throw new plugin_ui_utils_1.RequestError('Auth Initiation Not Found', new Error('Auth Initiation Not Found'));
            }
            try {
                const { AuthenticationResult } = yield (0, auth_1.getAuthTokens)(initiateAuthOutput, code);
                if (!AuthenticationResult) {
                    throw new Error('Incorrect Code');
                }
                const token = AuthenticationResult.RefreshToken;
                console.log('Generated token: ' + token);
                return { token };
            }
            catch (e) {
                console.error('Failed to generate token');
                console.error(e);
                throw new plugin_ui_utils_1.RequestError(e.message, e);
            }
        });
        this.onRequest('/send-code', this.generateCode);
        this.onRequest('/token', this.generateToken);
        this.ready();
    }
}
function startPluginUiServer() {
    return new PluginUiServer();
}
startPluginUiServer();
