"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PetSafeSmartFeedPlatform = void 0;
const api_1 = require("./api");
const hap_1 = require("./hap");
const util_1 = require("./util");
const accessory_1 = require("./accessory");
const { argv } = process, isTestHomebridge = argv.join(' ').includes('-P . -U ./.homebridge');
class PetSafeSmartFeedPlatform {
    constructor(log, config, api) {
        this.log = log;
        this.config = config;
        this.api = api;
        this.homebridgeAccessories = {};
        (0, util_1.useLogger)({
            logInfo(message) {
                log.info(message);
            },
            logError(message) {
                log.error(message);
            },
        });
        if (!config) {
            this.log.info('No configuration found for platform PetSafeSmartFeed');
            return;
        }
        this.api.on('didFinishLaunching', () => {
            this.log.debug('didFinishLaunching');
            this.connectToApi().catch((e) => {
                this.log.error('Error connecting to API');
                this.log.error(e);
            });
        });
        this.homebridgeAccessories = {};
    }
    configureAccessory(accessory) {
        this.log.info(`Configuring cached accessory ${accessory.UUID} ${accessory.displayName}`);
        this.log.debug('%j', accessory);
        this.homebridgeAccessories[accessory.UUID] = accessory;
    }
    connectToApi() {
        return __awaiter(this, void 0, void 0, function* () {
            const smartFeedApi = new api_1.SmartFeedApi(this.config), feeders = yield smartFeedApi.getFeeders(), cachedAccessoryIds = Object.keys(this.homebridgeAccessories), platformAccessories = [], activeAccessoryIds = [], debugPrefix = isTestHomebridge ? 'TEST ' : '', feederConfigs = this.config.feeders || [];
            this.log.info(`Configuring ${feeders.length} PetSafe Smart Feeder(s):`);
            feeders.forEach((feeder) => {
                let isNewFeeder = false;
                const uuid = hap_1.hap.uuid.generate(debugPrefix + feeder.id), displayName = debugPrefix + feeder.name, createHomebridgeAccessory = () => {
                    isNewFeeder = true;
                    const accessory = new this.api.platformAccessory(displayName, uuid, 5 /* hap.Categories.LIGHTBULB */);
                    platformAccessories.push(accessory);
                    return accessory;
                }, homebridgeAccessory = this.homebridgeAccessories[uuid] || createHomebridgeAccessory(), feederConfig = feederConfigs.find((config) => config.id === feeder.id);
                this.log.info(` - ${feeder.name} - ID: ${feeder.id}, Amount: ${(feederConfig === null || feederConfig === void 0 ? void 0 : feederConfig.amount)
                    ? feederConfig.amount + '/8 Cup'
                    : 'Repeat Last Meal'} ${isNewFeeder ? ' (NEW)' : ''}`);
                new accessory_1.SmartFeedAccessory(feeder, homebridgeAccessory, feederConfig);
                this.homebridgeAccessories[uuid] = homebridgeAccessory;
                activeAccessoryIds.push(uuid);
            });
            feederConfigs.forEach((config) => {
                if (config.id && !feeders.some((feeder) => feeder.id === config.id)) {
                    this.log.error(` - Config found with no matching feeder ID ${config.id}.  Double check that you have the right ID from the logs above.`);
                }
            });
            if (platformAccessories.length) {
                this.api.registerPlatformAccessories(hap_1.pluginName, hap_1.platformName, platformAccessories);
            }
            const staleAccessories = cachedAccessoryIds
                .filter((cachedId) => !activeAccessoryIds.includes(cachedId))
                .map((id) => this.homebridgeAccessories[id]);
            staleAccessories.forEach((staleAccessory) => {
                this.log.info(`Removing stale cached accessory ${staleAccessory.UUID} ${staleAccessory.displayName}`);
            });
            if (staleAccessories.length) {
                this.api.unregisterPlatformAccessories(hap_1.pluginName, hap_1.platformName, staleAccessories);
            }
        });
    }
}
exports.PetSafeSmartFeedPlatform = PetSafeSmartFeedPlatform;
