"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestClient = exports.requestWithRetry = exports.apiPath = void 0;
const got_1 = __importDefault(require("got"));
const util_1 = require("./util");
const auth_1 = require("./auth");
const apiBaseUrl = 'https://platform.cloud.petsafe.net/smart-feed/', defaultRequestOptions = {
    responseType: 'json',
    method: 'GET',
};
function apiPath(path) {
    return apiBaseUrl + path;
}
exports.apiPath = apiPath;
function requestWithRetry(options) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = (yield (0, got_1.default)(Object.assign(Object.assign({}, defaultRequestOptions), options)));
            return response.body;
        }
        catch (e) {
            if (!e.response) {
                (0, util_1.logError)(`Failed to reach PetSafe server at ${options.url}.  ${e.message}.  Trying again in 5 seconds...`);
                yield (0, util_1.delay)(5000);
                return requestWithRetry(options);
            }
            throw e;
        }
    });
}
exports.requestWithRetry = requestWithRetry;
class RestClient {
    constructor(authOptions) {
        this.authOptions = authOptions;
    }
    updateIdToken() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const { AuthenticationResult } = yield (0, auth_1.refreshTokens)(this.authOptions.token), idToken = AuthenticationResult === null || AuthenticationResult === void 0 ? void 0 : AuthenticationResult.IdToken, expiresInSeconds = AuthenticationResult === null || AuthenticationResult === void 0 ? void 0 : AuthenticationResult.ExpiresIn;
                this.idToken = idToken;
                if (expiresInSeconds && idToken) {
                    setTimeout(() => {
                        if (this.idToken === idToken) {
                            this.idToken = undefined;
                            this.updateIdToken();
                        }
                    }, (expiresInSeconds - 10) * 1000);
                }
            }
            catch (e) {
                (0, util_1.logError)(e.message);
            }
            if (!this.idToken) {
                throw new Error('Your PetSafe token is invalid or expired.  Please generate a new token and update your config.');
            }
        });
    }
    request(options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.idToken) {
                yield this.updateIdToken();
            }
            if (!this.idToken) {
                throw new Error('Unable to update auth token');
            }
            try {
                const headers = Object.assign(Object.assign({}, options.headers), { Authorization: this.idToken }), response = yield requestWithRetry(Object.assign(Object.assign({}, options), { headers }));
                return response;
            }
            catch (e) {
                const response = e.response || {}, { url } = options;
                if (response.status === 401) {
                    yield this.updateIdToken();
                    return this.request(options);
                }
                if (response.status === 404 && url.startsWith(apiBaseUrl)) {
                    (0, util_1.logError)('404 from endpoint ' + url);
                    throw new Error('Not found with response: ' + JSON.stringify(response.data));
                }
                (0, util_1.logError)(`Request to ${url} failed (${response.status}). ${JSON.stringify(response.data)}`);
                throw e;
            }
        });
    }
}
exports.RestClient = RestClient;
